/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is autoprint.
 *
 * The Initial Developer of the Original Code is
 * Daniel Luttinger.
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *		Daniel Luttinger <daniel@luttinger.eu>
 *		Frank Nagel <f.nagel@fkn-systems.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

(function autoPrint2(){
	
	// handy constants
	const Cc = Components.classes;
	const Ci = Components.interfaces;
	const Cu = Components.utils;
		
	// global scope variables and shorthand
	this.autoprint2 = {};
	var ap2 = this.autoprint2;
	
	// debug info and log shorthand
	ap2.version = "($VER: autoprint.js 2.2.1 (20110315 Daniel) (20110314 Daniel) Daniel,Frank $)";
	ap2.debug=false; // toggle log in javascript console
	ap2.log = function(str) {
		Application.console.log("ap2: "+str);
	};
		
	// our data heart
	ap2.running=false;
	ap2.timeout=null;
	ap2.msgIDList = [];
	ap2.folderUriList = [];
	
	
	// toggles the silent printing feature
	ap2.silent = function(bState) {
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("");
		prefs.setBoolPref("print.always_print_silent",bState); // do not ask to select printer
		prefs.setBoolPref("print.show_print_progress",!bState); // do not show a progress bar
		if(ap2.debug) {ap2.log("Set prefs.js print.always_print_silent: " + bState + "\n");};
	};
	ap2.silentOn = function() { ap2.silent(true);};
	ap2.silentOff = function() { ap2.silent(false);};
		
	// this function does the printing
	ap2.doPrint = function() {			
		if(!ap2.running) {return;};
		if(ap2.debug) {ap2.log("Searching my mails to print ... ");}
		if(ap2.msgIDList.length != ap2.folderUriList.length) {
			var msg = "PANIC: msgIDList("+ap2.msgIDList.length;
			msg +=  ") != folderUriList("+ap2.folderUriList.length+")";
			ap2.log(msg)
			ap2.running=false;
			msgIDList = [];
			folderUriList = [];
			return;
		}
		
		var count =	ap2.msgIDList.length; // number of mails to print
		var msgUriList = []; // to collect the URIs
		var isError = false; // flag for DB inconsistence
		
		if(ap2.debug) {ap2.log("getting " + count+" mails");}
		
		// iterate over messages to print
		for(var i=0; i < count;i++) { 
			var db = null;
			var fold = GetMsgFolderFromUri(ap2.folderUriList[i]);
			if(fold) {
				// if msf is corrupt, following will raise exception
				try {
					db = fold.msgDatabase;
					if(db) {
						hdr = db.getMsgHdrForMessageID(ap2.msgIDList[i]);  
						if(hdr) {
							if(ap2.debug) {ap2.log("MAIL FOUND!\nSubject: "+hdr.mime2DecodedSubject);};
							msgUriList.push(hdr.folder.getUriForMsg(hdr));
						}
					}
				}catch(e) {
					var emsg = fold.server.prettyName+"::"+fold.prettyName+"\n"+ap2.errorMsgMsf;
					Application.console.log(emsg); // always log to console on exception
					Application.console.log(e);
					if(!isError) alert(emsg); //display alert only once
					isError=true;
				}
			}
		}
		
		// to hide the dialog
		ap2.silentOn();
		
		// do the print magic
		if(ap2.debug) {ap2.log("Printing " + msgUriList.length+" mails");};
		window.openDialog("chrome://messenger/content/msgPrintEngine.xul", "","chrome,dialog=no,all,centerscreen",msgUriList.length, msgUriList, statusFeedback,false, Ci.nsIMsgPrintEngine.MNAB_PRINT_MSG, window);
		
		// when an printer-requester appears between multiprint increment this timer. Standart: '15000' # eq 15sec
		window.setTimeout(autoprint2.silentOff,count*4500); // switch off silent printing in mailcount * 4.5sec
		
		ap2.running=false;
		ap2.msgIDList = [];
		ap2.folderUriList = [];
	};
	//
	ap2.init = function() {
		if(ap2.debug) { ap2.log(ap2.version);}
		
		ap2.stringbundle = document.getElementById("autoprint2stringbundle");
		ap2.errorMsgMsf = ap2.stringbundle.getString('errorMsgMsf');
		
		this.printing = {
			id: "autoprint2@daniel.luttinger.eu#print",
			name: ap2.stringbundle.getString('printActionName'),
			apply: function(aMsgHdrs, aActionValue, aListener, aType, aMsgWindow) {
				if(!ap2.running) {
					ap2.running=true;
				}else{
					//reset the timeout if already started
					window.clearTimeout(ap2.timeout);
				}

				var count = aMsgHdrs.length; // number of msgs from filter
				var msg = ''; // holds debug strings

				for (var i = 0; i < count; i++) {
					var hdr = aMsgHdrs.queryElementAt(i, Ci.nsIMsgDBHdr);
					var uri = aActionValue=="none"?hdr.folder.URI:aActionValue;
					ap2.msgIDList.push(hdr.messageId);
					ap2.folderUriList.push(uri);
					msg += "ID: " + hdr.messageId + "\n";
					msg += "folder: " + uri + "\n";
				}
				if(ap2.debug) {ap2.log(msg);};
				
				// print async to collect multiple arriving at a time.
				ap2.timeout=window.setTimeout(autoprint2.doPrint,4000);
			},
			isValidForType: function(type, scope) {return true;},
			validateActionValue: function(value, folder, type) {return null;},
			allowDuplicates: false,needsBody: true
		};
		
		// here we get the filterservice and add our custom action
		var filterService = Cc["@mozilla.org/messenger/services/filters;1"].getService(Ci.nsIMsgFilterService);
		filterService.addCustomAction(printing);

		removeEventListener("load",autoprint2.init, true);
	}
})();
//
window.addEventListener("load", autoprint2.init, true);
//
